emacs-orgmode@gnu.org archives
 help / color / mirror / code / Atom feed
From: Gary Oberbrunner <garyo@oberbrunner.com>
To: Orgmode Mailing List <emacs-orgmode@gnu.org>
Subject: Re: patch for ob-sql.el: improve MySQL header formatting, and add db spec header vars
Date: Fri, 1 Feb 2013 13:57:46 -0500	[thread overview]
Message-ID: <CAFChFygWZ9GqgZR8qH133AXwamut6gx0PfaQjxqB2RY5bFNbaw@mail.gmail.com> (raw)
In-Reply-To: <CAFChFyjc2_tF=2TbCXF8mJUvqQ4qVC85TPL3Dq+WTaBAvmLvEA@mail.gmail.com>


[-- Attachment #1.1: Type: text/plain, Size: 901 bytes --]

Here's a better version of this patch, which also adds support for
:colnames, which I needed.  And it adds some doc about what header args are
used.

 * add a header-row delimiter to the tables returned from mysql
 * adds new sql-specific header args for the database connection, and
implements them for mysql
 * adds support for :colnames (mysql only)
 * (minor) adds an edebug spec to org-babel-result-cond to allow edebugging
through it



On Fri, Feb 1, 2013 at 10:18 AM, Gary Oberbrunner <garyo@oberbrunner.com>wrote:

> Let me know if this would be better as a pull request.  This patch does
> three things:
>
>  * add a header-row delimiter to the tables returned from mysql
>  * adds new sql-specific header args for the database connection, and
> implements them for mysql
>  * (minor) adds an edebug spec to org-babel-result-cond to allow
> edebugging through it
>
> --
> Gary
>



-- 
Gary

[-- Attachment #1.2: Type: text/html, Size: 1732 bytes --]

[-- Attachment #2: org-mode-sql-2.patch --]
[-- Type: application/octet-stream, Size: 5326 bytes --]

diff --git a/lisp/ob-core.el b/lisp/ob-core.el
index bdf8c54..1a42965 100644
--- a/lisp/ob-core.el
+++ b/lisp/ob-core.el
@@ -2585,7 +2585,8 @@ Emacs shutdown."))
 
 (defmacro org-babel-result-cond (result-params scalar-form &rest table-forms)
   "Call the code to parse raw string results according to RESULT-PARAMS."
-  (declare (indent 1))
+  (declare (indent 1)
+	   (debug (form form &rest form)))
   `(unless (member "none" ,result-params)
      (if (or (member "scalar" ,result-params)
 	     (member "verbatim" ,result-params)
diff --git a/lisp/ob-sql.el b/lisp/ob-sql.el
index 8d2ba24..51cafe6 100644
--- a/lisp/ob-sql.el
+++ b/lisp/ob-sql.el
@@ -32,12 +32,24 @@
 ;;
 ;; Also SQL evaluation generally takes place inside of a database.
 ;;
-;; For now lets just allow a generic ':cmdline' header argument.
+;; Header args used:
+;; - engine
+;; - cmdline
+;; - dbhost
+;; - dbuser
+;; - dbpassword
+;; - database
+;; - colnames (default, nil, means "yes")
+;; - result-params
+;; - out-file
+;; The following are used but not really implemented for SQL:
+;; - colname-names
+;; - rownames
+;; - rowname-names
 ;;
 ;; TODO:
 ;;
 ;; - support for sessions
-;; - add more useful header arguments (user, passwd, database, etc...)
 ;; - support for more engines (currently only supports mysql)
 ;; - what's a reasonable way to drop table data into SQL?
 ;;
@@ -52,21 +64,40 @@
 
 (defvar org-babel-default-header-args:sql '())
 
-(defvar org-babel-header-args:sql
-  '((engine   . :any)
-    (out-file . :any)))
+(defconst org-babel-header-args:sql
+  '((engine	       . :any)
+    (out-file	       . :any)
+    (dbhost	       . :any)
+    (dbuser	       . :any)
+    (dbpassword	       . :any)
+    (database	       . :any))
+  "SQL-specific header arguments.")
 
 (defun org-babel-expand-body:sql (body params)
   "Expand BODY according to the values of PARAMS."
   (org-babel-sql-expand-vars
    body (mapcar #'cdr (org-babel-get-header params :var))))
 
+(defun dbstring-mysql (host user password database)
+  "Make MySQL cmd line args for database connection.  Pass nil to omit that arg."
+  (combine-and-quote-strings
+   (remq nil
+	 (list (when host     (concat "-h" host))
+	       (when user     (concat "-u" user))
+	       (when password (concat "-p" password))
+	       (when database (concat "-D" database))))))
+
 (defun org-babel-execute:sql (body params)
   "Execute a block of Sql code with Babel.
 This function is called by `org-babel-execute-src-block'."
   (let* ((result-params (cdr (assoc :result-params params)))
          (cmdline (cdr (assoc :cmdline params)))
+         (dbhost (cdr (assoc :dbhost params)))
+         (dbuser (cdr (assoc :dbuser params)))
+         (dbpassword (cdr (assoc :dbpassword params)))
+         (database (cdr (assoc :database params)))
          (engine (cdr (assoc :engine params)))
+         (colnames-p (not (equal "no" (cdr (assoc :colnames params)))))
          (in-file (org-babel-temp-file "sql-in-"))
          (out-file (or (cdr (assoc :out-file params))
                        (org-babel-temp-file "sql-out-")))
@@ -85,7 +116,9 @@ This function is called by `org-babel-execute-src-block'."
                                      (or cmdline "")
                                      (org-babel-process-file-name in-file)
                                      (org-babel-process-file-name out-file)))
-                    ('mysql (format "mysql %s < %s > %s"
+                    ('mysql (format "mysql %s %s %s < %s > %s"
+				    (dbstring-mysql dbhost dbuser dbpassword database)
+				    (if colnames-p "" "-N")
                                     (or cmdline "")
 				    (org-babel-process-file-name in-file)
 				    (org-babel-process-file-name out-file)))
@@ -107,19 +140,34 @@ This function is called by `org-babel-execute-src-block'."
       (with-temp-buffer
 	  (progn (insert-file-contents-literally out-file) (buffer-string)))
       (with-temp-buffer
-	;; need to figure out what the delimiter is for the header row
-	(with-temp-buffer
-	  (insert-file-contents out-file)
-	  (goto-char (point-min))
-	  (when (re-search-forward "^\\(-+\\)[^-]" nil t)
-	    (setq header-delim (match-string-no-properties 1)))
-	  (goto-char (point-max))
-	  (forward-char -1)
-	  (while (looking-at "\n")
-	    (delete-char 1)
-	    (goto-char (point-max))
-	    (forward-char -1))
-	  (write-file out-file))
+	(case (intern engine)
+	  ('mysql
+	   ;; add header row delimiter after column-names header in first line
+	   (cond
+	    (colnames-p
+	     (with-temp-buffer
+	       (insert-file-contents out-file)
+	       (goto-char (point-min))
+	       (forward-line 1)
+	       (insert "-\n")
+	       (setq header-delim "-")
+	       (write-file out-file)
+	       ))))
+	  (t
+	   ;; need to figure out what the delimiter is for the header row
+	   (with-temp-buffer
+	     (insert-file-contents out-file)
+	     (goto-char (point-min))
+	     (when (re-search-forward "^\\(-+\\)[^-]" nil t)
+	       (setq header-delim (match-string-no-properties 1)))
+	     (goto-char (point-max))
+	     (forward-char -1)
+	     (while (looking-at "\n")
+	       (delete-char 1)
+	       (goto-char (point-max))
+	       (forward-char -1))
+	     (write-file out-file)))
+	  )
 	(org-table-import out-file '(16))
 	(org-babel-reassemble-table
 	 (mapcar (lambda (x)

  reply	other threads:[~2013-02-01 19:05 UTC|newest]

Thread overview: 3+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2013-02-01 15:18 patch for ob-sql.el: improve MySQL header formatting, and add db spec header vars Gary Oberbrunner
2013-02-01 18:57 ` Gary Oberbrunner [this message]
2013-02-03 22:53   ` Eric Schulte

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

  List information: https://www.orgmode.org/

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=CAFChFygWZ9GqgZR8qH133AXwamut6gx0PfaQjxqB2RY5bFNbaw@mail.gmail.com \
    --to=garyo@oberbrunner.com \
    --cc=emacs-orgmode@gnu.org \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
Code repositories for project(s) associated with this public inbox

	https://git.savannah.gnu.org/cgit/emacs/org-mode.git

This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox;
as well as URLs for read-only IMAP folder(s) and NNTP newsgroup(s).